# Generated by Django 2.2.24 on 2021-10-29 13:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edx_proctoring', '0020_auto_20211028_1915'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='proctoredexamsoftwaresecurereview',
            name='video_url',
        ),
        migrations.RemoveField(
            model_name='proctoredexamsoftwaresecurereviewhistory',
            name='video_url',
        ),
        migrations.AddField(
            model_name='proctoredexamsoftwaresecurereviewhistory',
            name='encrypted_video_url',
            field=models.BinaryField(null=True),
        ),
    ]



"""
Tests for the update_attempt_status_from_review management command
"""
from datetime import datetime, timedelta

import pytz

from django.core.management import call_command

from edx_proctoring.api import create_exam, create_exam_attempt, get_exam_attempt_by_id
from edx_proctoring.models import ProctoredExamSoftwareSecureReview
from edx_proctoring.runtime import set_runtime_service
from edx_proctoring.statuses import ProctoredExamStudentAttemptStatus, SoftwareSecureReviewStatus
from edx_proctoring.tests.test_services import MockCertificateService, MockCreditService, MockGradesService
from edx_proctoring.tests.utils import LoggedInTestCase


class SetAttemptActiveFieldTests(LoggedInTestCase):
    """
    Coverage of the update_attempt_status_from_review.py file
    """

    def setUp(self):
        """
        Build up test data
        """
        super().setUp()
        set_runtime_service('credit', MockCreditService())
        set_runtime_service('grades', MockGradesService())
        set_runtime_service('certificates', MockCertificateService())
        self.first_exam_id = create_exam(
            course_id='a/b/c',
            content_id='bar',
            exam_name='Test Exam 1',
            time_limit_mins=90
        )
        self.second_exam_id = create_exam(
            course_id='a/b/c',
            content_id='baz',
            exam_name='Test Exam 2',
            time_limit_mins=90
        )

        # create a first attempt and review
        self.first_attempt_id = create_exam_attempt(
            self.first_exam_id,
            self.user.id,
            taking_as_proctored=True
        )
        self.first_attempt = get_exam_attempt_by_id(self.first_attempt_id)
        self.first_attempt_review_object = ProctoredExamSoftwareSecureReview.objects.create(
            attempt_code=self.first_attempt['attempt_code'],
            exam_id=self.first_exam_id,
            student_id=self.user.id,
        )

        # create a second attempt and review
        self.second_attempt_id = create_exam_attempt(
            self.second_exam_id,
            self.user.id,
            taking_as_proctored=True
        )
        self.second_attempt = get_exam_attempt_by_id(self.second_attempt_id)
        self.second_attempt_review_object = ProctoredExamSoftwareSecureReview.objects.create(
            attempt_code=self.second_attempt['attempt_code'],
            exam_id=self.second_exam_id,
            student_id=self.user.id,
        )

        # update both reviews, and use update instead of save to avoid triggering a post_save signal
        ProctoredExamSoftwareSecureReview.objects.filter(id=self.first_attempt_review_object.id).update(
            review_status=SoftwareSecureReviewStatus.clean
        )
        ProctoredExamSoftwareSecureReview.objects.filter(id=self.second_attempt_review_object.id).update(
            review_status=SoftwareSecureReviewStatus.suspicious
        )

    def test_run_command(self):
        """
        Run the management command
        """
        # check status of attempts
        self.assertEqual(self.first_attempt['status'], ProctoredExamStudentAttemptStatus.created)
        self.assertEqual(self.second_attempt['status'], ProctoredExamStudentAttemptStatus.created)
        # check status of reviews
        first_review = ProctoredExamSoftwareSecureReview.objects.get(id=self.first_attempt_review_object.id)
        self.assertEqual(first_review.review_status, SoftwareSecureReviewStatus.clean)
        second_review = ProctoredExamSoftwareSecureReview.objects.get(id=self.second_attempt_review_object.id)
        self.assertEqual(second_review.review_status, SoftwareSecureReviewStatus.suspicious)

        start_time = datetime.now(pytz.UTC) - timedelta(minutes=60)
        end_time = datetime.now(pytz.UTC) + timedelta(minutes=60)

        # run command
        call_command(
            'update_attempt_status_from_review',
            batch_size=2,
            sleep_time=0,
            start_date_time=start_time.strftime('%Y-%m-%d %H:%M:%S'),
            end_date_time=end_time.strftime('%Y-%m-%d %H:%M:%S')
        )

        # check that attempt statuses has been updated
        updated_first_attempt = get_exam_attempt_by_id(self.first_attempt_id)
        self.assertEqual(updated_first_attempt['status'], ProctoredExamStudentAttemptStatus.verified)
        updated_second_attempt = get_exam_attempt_by_id(self.second_attempt_id)
        self.assertEqual(updated_second_attempt['status'], ProctoredExamStudentAttemptStatus.second_review_required)



"""
Django management command to update the is_attempt_active field on
ProctoredExamSoftwareSecureReview and ProctoredExamSoftwareSecureReviewHistory models
"""
import logging
import time

from django.core.management.base import BaseCommand

from edx_proctoring.models import (
    ProctoredExamSoftwareSecureReview,
    ProctoredExamSoftwareSecureReviewHistory,
    ProctoredExamStudentAttempt
)

log = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Django Management command to update is_attempt_active field on review models
    """
    update_field_count = 0
    update_attempt_codes = []
    distinct_attempt_codes = set()

    def add_arguments(self, parser):
        parser.add_argument(
            '--batch_size',
            action='store',
            dest='batch_size',
            type=int,
            default=300,
            help='Maximum number of attempt_codes to process. '
                 'This helps avoid locking the database while updating large amount of data.'
        )
        parser.add_argument(
            '--sleep_time',
            action='store',
            dest='sleep_time',
            type=int,
            default=10,
            help='Sleep time in seconds between update of batches'
        )

    def handle(self, *args, **options):
        """
        Management command entry point, simply call into the signal firing
        """

        batch_size = options['batch_size']
        sleep_time = options['sleep_time']

        log.info('Updating is_attempt_active field for current reviews.')
        for review in ProctoredExamSoftwareSecureReview.objects.filter(is_attempt_active=True):
            self.check_and_update(review, batch_size, sleep_time)

        log.info('Updating is_attempt_active field for archived reviews.')
        for archived_review in ProctoredExamSoftwareSecureReviewHistory.objects.filter(is_attempt_active=True):
            self.check_and_update(archived_review, batch_size, sleep_time, only_update_archives=True)

        if self.update_attempt_codes:
            log.info('Updating %i reviews', len(self.update_attempt_codes))
            self.bulk_update(self.update_attempt_codes, False)

    def check_and_update(self, review_object, size, sleep_time, only_update_archives=False):
        """
        Function to check if a review object should be updated, and updates accordingly
        """
        if review_object.attempt_code not in self.distinct_attempt_codes:
            if self.should_update(review_object):
                self.distinct_attempt_codes.add(review_object.attempt_code)
                self.update_attempt_codes.append(review_object.attempt_code)
                self.update_field_count += 1
                log.info('Adding review %i to be updated', review_object.id)

            if self.update_field_count == size:
                log.info('Updating %i reviews', size)
                self.bulk_update(self.update_attempt_codes, only_update_archives)
                self.update_field_count = 0
                self.update_attempt_codes = []
                time.sleep(sleep_time)

    def bulk_update(self, attempt_codes, only_update_archive):
        """
        Updates the is_attempt_active fields for all reviews who have an attempt code in attempt_codes
        """
        if not only_update_archive:
            reviews = ProctoredExamSoftwareSecureReview.objects.filter(attempt_code__in=attempt_codes)
            reviews.update(is_attempt_active=False)

        archived_reviews = ProctoredExamSoftwareSecureReviewHistory.objects.filter(attempt_code__in=attempt_codes)
        archived_reviews.update(is_attempt_active=False)

    def should_update(self, review_object):
        """
        Returns a boolean based on whether an attempt exists in the ProctoredExamStudentAttempt model
        """
        attempt_code = review_object.attempt_code
        try:
            ProctoredExamStudentAttempt.objects.get(attempt_code=attempt_code)
            return False
        except ProctoredExamStudentAttempt.DoesNotExist:
            return True



"""
URL mapping for the exam instructor dashboard.
"""

from django.conf import settings
from django.urls import re_path

from edx_proctoring import views

app_name = 'instructor'


urlpatterns = [
    re_path(
        fr'edx_proctoring/v1/instructor/{settings.COURSE_ID_PATTERN}/(?P<exam_id>\d+)$',
        views.InstructorDashboard.as_view(),
        name='instructor_dashboard_exam'
    ),
]
